
/* Chrysalide - Outil d'analyse de fichiers binaires
 * debugger-int.h - prototypes pour l'interface des débogueurs
 *
 * Copyright (C) 2009-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DEBUG_DEBUGGER_INT_H
#define _DEBUG_DEBUGGER_INT_H


#include "debugger.h"


#include <gtk/gtk.h>



/* Procède au démarrage effectif de la session de débogage. */
typedef bool (* attach_debugger_fc) (GBinaryDebugger *);

/* Démarre, met en pause ou tue une procédure de débogage. */
typedef bool (* basic_debugger_fc) (GBinaryDebugger *);

/* Reprend une procédure de débogage. */
typedef bool (* resume_debugger_fc) (GBinaryDebugger *);

/* Fournit les identifiants de tous les threads actifs. */
typedef pid_t * (* dbg_list_all_threads_fc) (GBinaryDebugger *, char ***, size_t *);

/* Fournit la liste des frames courantes d'un thread donné. */
typedef dbg_frame_t * (* dbg_get_frames_stack_fc) (GBinaryDebugger *, pid_t, size_t *);

/* Fournit la valeur des registres de l'architecture. */
typedef register_value * (* get_register_values_fc) (GBinaryDebugger *, size_t *);


/* Définition des fonctionnalités d'un débogueur (instance) */
struct _GBinaryDebugger
{
    GObject parent;                         /* A laisser en premier        */

    GLoadedBinary *binary;                  /* Cible à traiter             */

    attach_debugger_fc attach;              /* Démarre le débogueur        */

    basic_debugger_fc run;                  /* Démarre le débogueur        */
    basic_debugger_fc pause;                /* Met en pause le débogueur   */
    resume_debugger_fc resume;              /* Relance le débogueur        */
    basic_debugger_fc kill;                 /* Tue le débogueur            */

    dbg_list_all_threads_fc all_threads;    /* Liste des threads actifs    */
    dbg_get_frames_stack_fc frames_stack;   /* Pile des frames courantes   */

    get_register_values_fc get_reg_values;  /* Obtient les valeurs de reg. */

};


/* Définition des fonctionnalités d'un débogueur (classe) */
struct _GBinaryDebuggerClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    /* Signaux */

    void (* debugger_stopped) (GBinaryDebugger *, uint64_t, uint64_t);

    void (* debugger_halted) (GBinaryDebugger *, int, vmpa_t, pid_t);

};


#endif  /* _DEBUG_DEBUGGER_INT_H */
